--=============================================================================
-------------------------Example SDevPlugin Lua Script-------------------------
-------------------- Draws a rendering of the Mandelbrot set-------------------
--=============================================================================


SLua.SLogger_global():addMessage(">>Mandlebrotter v0.1<<")

-- set up some convenient input variables
local inputs = {...}
local target = inputs[3]
local plane_size = SLua.SCoordinate(2):xy(1920,1080)
-- configure the include path, properly
local include_path = SLua.SURI(SLua.sysInfo_simulacrumDIR())
include_path:addComponentBack(SLua.SPluginManager_includesPath())

SLua.SLogger_global():addMessage(">>Include @ " .. include_path:getURI())

-- configure container space
target:resize(plane_size);

-- mandelbrot_at --------------------------------------------------------------
-- Returns the iteration depth at which 
function mandelbrot_at(baser, basei, r, i,depth, maxdepth)
  mandel_thresh = 4.0 -- using 4, no need to sqrt below
  local mag = (r*r) + (i*i)
  if (mag >= mandel_thresh) or (depth >=maxdepth) then
    return depth
  else
    -- (a+bi).(c+di) = (ac-bd) + (bc+ad)i
    local newr = ( (r*r) - (i*i) ) + baser
    local newi = ( (i*r) * 2 )     + basei
    return mandelbrot_at(baser,basei,newr,newi,depth+1,maxdepth)
  end
end ------------------------------------------------------------- mandelbrot_at

-- plot_mandelbrot ------------------------------------------------------------
function plot_mandelbrot(target_space, depth)
  target_space:progress(-1) -- trigger progress indication
  target_space:lock() -- stop external modifications
  -- configure the space for RGB data
  target_space:setName("Mandelbrot") -- give it a name
  -- create an instance of an elem
  local elemtype = SLua.BW8SElem(SLua.SElem_nullSource())
  target_space:setNativeSElemType(elemtype:New()) -- use it for the whole space
  -- set a LUT
  target_space:LUT():useWL(true);
  target_space:LUT():genLUT(8,depth/2,depth);
  -- fetch extent
  local extent = target_space:extent()
  local dims   = extent:getDim()
  SLua.SLogger_global():addMessage(">>Starting @ res " .. extent:toString()
                                 .. " @ depth " .. depth)
  if dims == 2 then -- safety checks
    --configure the mandelbrot view area
    local xinc =  3.0 / extent:x() -- scale x
    local yinc =  2.0 / extent:y() -- scale y
    local xoff = -2.0  -- xoffset
    local yoff =  1.0  -- yoffset
    -- iterate on the space
    SLua.SLogger_global():addMessage(">>Processing...")
    local iter = target_space:getFastNativeIterator() -- use an interator
    while not iter:isEnd() do
      iterpos = iter:pos()
      local xpos = xoff + (xinc * iterpos:x()) 
      local ypos = yoff - (yinc * iterpos:y())
      iter:selem():intensity(mandelbrot_at(xpos,ypos,0,0,0,depth))
      iter:increment()
    end
  else
    SLua.SLogger_global():addMessage("!!Wrong number of dims: " .. dims)
  end
  target_space:unlock() -- release it
  target_space:progress(100) -- signal completion
  target_space:refresh(true) -- trigger update
end ----------------------------------------------------------- plot_mandelbrot

-- call plotter on target space
plot_mandelbrot(target, 128)
